
package com.example.acmecorp;

import java.util.Arrays;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.Bean;
import org.springframework.data.neo4j.core.Neo4jClient;
import org.springframework.data.neo4j.repository.config.EnableNeo4jRepositories;
import org.springframework.util.Assert;

@SpringBootApplication
@EnableNeo4jRepositories
public class AcmeCorpApplication {

	private final static Logger log = LoggerFactory.getLogger(AcmeCorpApplication.class);

	public static void main(String[] args) throws Exception {
		SpringApplication.run(AcmeCorpApplication.class, args);
		System.exit(0);
	}

	@Bean
	CommandLineRunner demo( CompanyRepository companyRepository, Neo4jClient neo4jClient) {
		return args -> {

			// clean up
			neo4jClient.query("match (n) detach delete n").run();

			// setup the company and its employees
			// sales persons
			Person greg = new Sales("Greg");
			Person roy = new Sales("Roy");
			Person craig = new Sales("Craig");

			// devs
			Language java = new Language("java", "1.5");
			Language perl = new Language("perl", "6.0");
			Developer harry = new Developer("Harry", List.of(java, perl));

			// setup and save the company
			List<Person> team = Arrays.asList(greg, roy, craig, harry);
			Company acme = new Company("ACME", team);
			companyRepository.save(acme);


			// read company from db
			Company loadedAcme = companyRepository.findByName("ACME");
			loadedAcme.getEmployees().forEach( e -> log.info(e.toString()));

			// find Harry...
			Developer loadedHarry = loadedAcme.getEmployees().stream()
					.filter(e -> e instanceof Developer)
					.map(e -> (Developer) e)
					.filter(developer -> developer.getName().equals("Harry"))
					.findFirst().get();

			// ... and check for harry's programming skills
			Assert.notEmpty(loadedHarry.getProgrammingLanguages(), "Harry forgot about all of his programming skills :(");

		};
	}

}
